@@ -173,18 +173,14 @@ imap_auth_login_check_not_STARTTLS (const gchar *response, GError **error)
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gchar *
-imap_auth_escape_backslash (const gchar *str)
+imap_auth_login_escape0 (const gchar *str)
 {
   GString *ret;
-  gsize i;
-  gsize len;
 
   ret = g_string_new ("");
-  len = strlen (str);
-
-  for (i = 0; i < len; i++)
+  for (size_t i = 0; str && str[i]; i++)
     {
-      if (str[i] == '\\')
+      if (str[i] == '\\' || str[i] == '"' || str[i] == '{' || str[i] == '}')
         g_string_append_c (ret, '\\');
       g_string_append_c (ret, str[i]);
     }
@@ -358,9 +354,11 @@ goa_imap_auth_login_run_sync (GoaMailAuth         *auth,
   gchar *request = NULL;
   gchar *response = NULL;
   gboolean ret = FALSE;
+  gchar *username = NULL;
   gchar *password = NULL;
 
-  password = imap_auth_escape_backslash (self->password);
+  username = imap_auth_login_escape0 (self->username);
+  password = imap_auth_login_escape0 (self->password);
 
   input = goa_mail_auth_get_input (auth);
   output = goa_mail_auth_get_output (auth);
@@ -382,7 +380,7 @@ goa_imap_auth_login_run_sync (GoaMailAuth         *auth,
 
   /* Send LOGIN */
 
-  request = g_strdup_printf ("%s LOGIN \"%s\" \"%s\"\r\n", IMAP_TAG, self->username, password);
+  request = g_strdup_printf ("%s LOGIN \"%s\" \"%s\"\r\n", IMAP_TAG, username, password);
   g_debug ("> %s LOGIN \"********************\" \"********************\"", IMAP_TAG);
   if (!g_data_output_stream_put_string (output, request, cancellable, error))
     goto out;
@@ -413,6 +411,7 @@ goa_imap_auth_login_run_sync (GoaMailAuth         *auth,
  out:
   g_free (response);
   g_free (request);
+  g_free (username);
   g_free (password);
   return ret;
 }
