@@ -172,7 +172,7 @@ get_identity_sync (GoaOAuth2Provider  *oauth2_provider,
   rest_proxy_call_set_method (call, "GET");
   rest_proxy_call_add_header (call, "Authorization", authorization);
 
-  if (!rest_proxy_call_sync (call, error))
+  if (!goa_rest_proxy_call_sync (call, cancellable, error))
     {
         goto out;
     }
@@ -229,9 +229,8 @@ get_identity_sync (GoaOAuth2Provider  *oauth2_provider,
     }
   id = json_node_get_string (json_member);
 
-  // Prefer "userPrincipalName" then "mail", failing if neither is available, since Gvfs requires
-  // a user and host for the GMountSpec
-  json_member = json_object_get_member (json_object, "userPrincipalName");
+  // Prefer "mail" then "displayName" for "PresentationIdentity", failing if neither is available
+  json_member = json_object_get_member (json_object, "mail");
   if (json_member != NULL && json_node_get_value_type (json_member) == G_TYPE_STRING)
     {
       presentation_identity = json_node_get_string (json_member);
@@ -239,15 +238,14 @@ get_identity_sync (GoaOAuth2Provider  *oauth2_provider,
 
   if (presentation_identity == NULL)
     {
-      json_member = json_object_get_member (json_object, "mail");
+      json_member = json_object_get_member (json_object, "displayName");
       if (json_member != NULL && json_node_get_value_type (json_member) == G_TYPE_STRING)
         {
           presentation_identity = json_node_get_string (json_member);
         }
       else
         {
-          g_debug ("%s(): expected \"userPrincipalName\" or \"mail\" field holding a string",
-                   G_STRFUNC);
+          g_debug ("%s(): expected \"mail\" or \"displayName\" field holding a string", G_STRFUNC);
           g_set_error (error,
                        GOA_ERROR,
                        GOA_ERROR_FAILED,
@@ -285,7 +283,7 @@ build_object (GoaProvider        *provider,
   GKeyFile *goa_conf;
   GoaMail *mail = NULL;
   const gchar  *provider_type;
-  const gchar *identity = NULL;
+  const gchar *presentation_identity = NULL;
   gboolean files_enabled = FALSE;
   gboolean calendar_enabled;
   gboolean contacts_enabled;
@@ -305,12 +303,12 @@ build_object (GoaProvider        *provider,
   provider_type = goa_provider_get_provider_type (provider);
   goa_conf = goa_util_open_goa_conf ();
   account = goa_object_get_account (GOA_OBJECT (object));
-  identity = goa_account_get_identity (account);
+  presentation_identity = goa_account_get_presentation_identity (account);
 
   /* Files */
   files_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_FILES) &&
                   g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
-  uri_onedrive = g_strconcat ("onedrive://", identity, "/", NULL);
+  uri_onedrive = g_strconcat ("onedrive://", presentation_identity, "/", NULL);
   goa_object_skeleton_attach_files (object, uri_onedrive, files_enabled, FALSE);
   g_free (uri_onedrive);
 
@@ -417,6 +415,8 @@ typedef struct
   GtkWidget *organization_switch;
   GtkWidget *client_id_entry;
   GtkWidget *tenant_id_entry;
+  GtkWidget *copy_button;
+  GoaAuthFlowFlags flags;
 } AccountData;
 
 static void
@@ -461,6 +461,13 @@ on_organization_changed (AdwSwitchRow *row,
     goa_provider_dialog_set_state (data->dialog, GOA_DIALOG_READY);
 }
 
+static void
+on_copylink_activated (AccountData *data)
+{
+  data->flags |= GOA_AUTH_FLOW_DO_NOT_LAUNCH_URI;
+  goa_provider_dialog_set_state (data->dialog, GOA_DIALOG_BUSY);
+}
+
 static void
 create_account_details_ui (GoaProvider *provider,
                            AccountData *data,
@@ -472,6 +479,7 @@ create_account_details_ui (GoaProvider *provider,
     {
       GtkWidget *group;
       GtkWidget *button;
+      GtkWidget *copy_desc;
 
       goa_provider_dialog_add_page (dialog,
                                     NULL, // provider name
@@ -494,9 +502,6 @@ create_account_details_ui (GoaProvider *provider,
       data->tenant_id_entry = goa_provider_dialog_add_entry (dialog, group, _("_Tenant ID"));
       goa_provider_dialog_add_description (dialog, data->tenant_id_entry, _("Example ID: 00000000-0000-0000-0000-000000000000"));
 
-      button = adw_dialog_get_default_widget (ADW_DIALOG (dialog));
-      gtk_button_set_label (GTK_BUTTON (button), _("_Sign In…"));
-
       g_object_bind_property (data->organization_switch, "active",
                               group, "visible",
                               G_BINDING_SYNC_CREATE);
@@ -513,6 +518,48 @@ create_account_details_ui (GoaProvider *provider,
                         G_CALLBACK (on_entry_changed),
                         data);
       on_organization_changed (ADW_SWITCH_ROW (data->organization_switch), NULL, data);
+
+      /* Auth Flow */
+      group = goa_provider_dialog_add_group (dialog, NULL);
+      adw_preferences_group_set_separate_rows (ADW_PREFERENCES_GROUP (group), TRUE);
+      gtk_widget_set_valign (group, GTK_ALIGN_END);
+      gtk_widget_set_vexpand (group, TRUE);
+
+      button = g_object_new (ADW_TYPE_BUTTON_ROW,
+                             "title", _("_Sign In…"),
+                             "use-underline", TRUE,
+                             NULL);
+      gtk_widget_add_css_class (button, "suggested-action");
+      adw_preferences_group_add (ADW_PREFERENCES_GROUP (group), button);
+
+      /* When "Copy Link" is clicked, we pin the flags for the auth flow */
+      data->copy_button = g_object_new (ADW_TYPE_BUTTON_ROW,
+                                        "title", _("_Copy Link"),
+                                        "use-underline", TRUE,
+                                        "start-icon-name", "edit-copy-symbolic",
+                                        NULL);
+      g_signal_connect_swapped (data->copy_button,
+                                "activated",
+                                G_CALLBACK (on_copylink_activated),
+                                data);
+      g_object_bind_property (button, "sensitive",
+                              data->copy_button, "sensitive",
+                              G_BINDING_DEFAULT);
+      adw_preferences_group_add (ADW_PREFERENCES_GROUP (group), data->copy_button);
+
+      copy_desc = gtk_label_new (_("Copy the authorization URL to continue with a specific web browser."));
+      gtk_label_set_justify (GTK_LABEL (copy_desc), GTK_JUSTIFY_CENTER);
+      gtk_label_set_wrap (GTK_LABEL (copy_desc), TRUE);
+      gtk_widget_set_halign (copy_desc, GTK_ALIGN_CENTER);
+      gtk_widget_set_margin_top (copy_desc, 18);
+      gtk_widget_add_css_class (copy_desc, "caption");
+      gtk_accessible_update_relation (GTK_ACCESSIBLE (data->copy_button),
+                                      GTK_ACCESSIBLE_RELATION_DESCRIBED_BY, copy_desc, NULL,
+                                      -1);
+      adw_preferences_group_add (ADW_PREFERENCES_GROUP (group), copy_desc);
+
+      /* Set the default widget after it's a child of the window */
+      adw_dialog_set_default_widget (ADW_DIALOG (dialog), button);
     }
 }
 
@@ -577,6 +624,9 @@ add_account_action_cb (GoaProviderDialog *dialog,
   self->redirect_uri = g_strdup_printf ("goa-oauth2://localhost/%s", client_id);
 
   /* With the provider configured for the client, we chain-up to authorize */
+  g_object_set_data (G_OBJECT (data->dialog),
+                     "goa-auth-flow-flags",
+                     GUINT_TO_POINTER (data->flags));
   GOA_PROVIDER_CLASS (goa_ms_graph_provider_parent_class)->add_account (provider,
                                                                         data->client,
                                                                         GTK_WIDGET (data->dialog),
