@@ -32,6 +32,10 @@
 #include "goaoauth2provider-priv.h"
 #include "goarestproxy.h"
 
+#define OAUTH2_DBUS_HANDLER_NAME  "org.gnome.OnlineAccounts.OAuth2"
+#define OAUTH2_DBUS_HANDLER_PATH  "/org/gnome/OnlineAccounts/OAuth2"
+#define OAUTH2_DBUS_HANDLER_IFACE "org.gnome.OnlineAccounts.OAuth2"
+
 /**
  * SECTION:goaoauth2provider
  * @title: GoaOAuth2Provider
@@ -68,6 +72,19 @@ G_DEFINE_ABSTRACT_TYPE (GoaOAuth2Provider, goa_oauth2_provider, GOA_TYPE_PROVIDE
 
 #define GOA_OAUTH2_CODE_CHALLENGE_METHOD_S256 "S256"
 
+/* Error Responses for Access Token Request
+ *
+ * See:
+ *  - https://datatracker.ietf.org/doc/html/rfc6749#section-5.2
+ *  - https://learn.microsoft.com/en-us/entra/identity-platform/v2-oauth2-auth-code-flow#error-codes-for-token-endpoint-errors
+ */
+#define GOA_OAUTH2_ACCESS_ERROR_INVALID_REQUEST         "invalid_request"
+#define GOA_OAUTH2_ACCESS_ERROR_INVALID_CLIENT          "invalid_client"
+#define GOA_OAUTH2_ACCESS_ERROR_INVALID_GRANT           "invalid_grant"
+#define GOA_OAUTH2_ACCESS_ERROR_UNAUTHORIZED_CLIENT     "unauthorized_client"
+#define GOA_OAUTH2_ACCESS_ERROR_UNAUTHORIZED_GRANT_TYPE "unauthorized_grant_type"
+#define GOA_OAUTH2_ACCESS_ERROR_INVALID_SCOPE           "invalid_scope"
+
 static gboolean
 is_authorization_error (GError *error)
 {
@@ -603,6 +620,7 @@ get_tokens_sync (GoaOAuth2Provider  *self,
                  GCancellable       *cancellable,
                  GError            **error)
 {
+  g_autoptr (GError) rest_error = NULL;
   GError *tokens_error = NULL;
   RestProxy *proxy;
   RestProxyCall *call;
@@ -641,12 +659,60 @@ get_tokens_sync (GoaOAuth2Provider  *self,
     if (data->code_verifier != NULL)
       rest_proxy_call_add_param (call, "code_verifier", data->code_verifier);
 
-  if (!goa_rest_proxy_call_sync (call, cancellable, error))
-    goto out;
-
-  status_code = rest_proxy_call_get_status_code (call);
-  if (status_code != 200)
+  if (!goa_rest_proxy_call_sync (call, cancellable, &rest_error))
     {
+      status_code = rest_proxy_call_get_status_code (call);
+      if (SOUP_STATUS_IS_CLIENT_ERROR (status_code))
+        {
+          g_autoptr (JsonParser) parser = NULL;
+          JsonObject *object;
+          const char *response_err;
+          const char *response_desc;
+          const char *response_suberr;
+
+          payload = rest_proxy_call_get_payload (call);
+          payload_length = rest_proxy_call_get_payload_length (call);
+
+          parser = json_parser_new ();
+          if (!json_parser_load_from_data (parser, payload, payload_length, &tokens_error))
+            {
+              g_warning ("json_parser_load_from_data() failed: %s (%s, %d)",
+                         tokens_error->message,
+                         g_quark_to_string (tokens_error->domain),
+                         tokens_error->code);
+              g_set_error (error,
+                           GOA_ERROR,
+                           GOA_ERROR_FAILED,
+                           _("Could not parse response"));
+              goto out;
+            }
+
+          object = json_node_get_object (json_parser_get_root (parser));
+          response_err = json_object_get_string_member_with_default (object, "error", NULL);
+          response_desc = json_object_get_string_member_with_default (object, "error_description", NULL);
+          response_suberr = json_object_get_string_member_with_default (object, "suberror", NULL);
+
+          if (response_suberr != NULL)
+            g_debug ("%s(): [%s/%s] %s", G_STRFUNC, response_err, response_suberr, response_desc);
+          else
+            g_debug ("%s(): [%s] %s", G_STRFUNC, response_err, response_desc);
+
+          /* Some OAuth providers have access policies controlled by the service. These cases may
+           * require re-authentication rather than simply refreshing.
+           */
+          if (g_strcmp0 (response_err, GOA_OAUTH2_ACCESS_ERROR_INVALID_GRANT) == 0)
+            {
+              g_set_error (error,
+                           GOA_ERROR,
+                           GOA_ERROR_NOT_AUTHORIZED,
+                           _("Authorization response: %s"),
+                           response_desc ? response_desc : rest_error->message);
+              goto out;
+            }
+        }
+
+      /* Any errors not known to require special handling return a generic error.
+       */
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
@@ -997,102 +1063,141 @@ create_account_details_ui (GoaProvider *provider,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static const SecretSchema oauth2_schema =
-{
-  .name = "org.gnome.OnlineAccounts.OAuth2",
-  .flags = SECRET_SCHEMA_NONE,
-  .attributes = {
-    {
-      .name = "goa-oauth2-client",
-      .type = SECRET_SCHEMA_ATTRIBUTE_STRING,
-    },
-    { "NULL", 0 }
-  }
-};
+static GDBusNodeInfo *oauth2_handler_dbus_info = NULL;
+static const char oauth2_handler_dbus_xml[] =
+  "<node>"
+  "  <interface name='"OAUTH2_DBUS_HANDLER_IFACE"'>"
+  "    <method name='Response'>"
+  "      <arg type='s' name='client-id' direction='in'/>"
+  "      <arg type='s' name='response' direction='in'/>"
+  "    </method>"
+  "  </interface>"
+  "</node>";
 
 typedef struct
 {
   char *request_uri;
   GoaAuthFlowFlags flags;
-  SecretCollection *collection;
   GCancellable *cancellable;
   unsigned long cancellable_id;
-  unsigned int completed : 1;
+
+  /* D-Bus */
+  GDBusConnection *connection;
+  unsigned int name_owner_id;
+  unsigned int register_object_id;
 } AuthorizeUriData;
 
+static void
+authorize_uri_task_complete (GCancellable *cancellable,
+                             GTask        *task)
+{
+  AuthorizeUriData *data = g_task_get_task_data (task);
+
+  g_assert (data != NULL);
+
+  /* If @cancellable is not %NULL, we're in the callback and the task needs to
+   * be completed manually. Otherwise, it's safe to disconnect the callback.
+   */
+  if (cancellable != NULL)
+    {
+      g_task_return_error_if_cancelled (task);
+    }
+  else if (data->cancellable != NULL && data->cancellable_id != 0)
+    {
+      g_cancellable_disconnect (data->cancellable, data->cancellable_id);
+      g_clear_object (&data->cancellable);
+    }
+
+  if (data->register_object_id != 0)
+    {
+      g_dbus_connection_unregister_object (data->connection, data->register_object_id);
+      data->register_object_id = 0;
+    }
+
+  if (data->name_owner_id != 0)
+    {
+      g_bus_unown_name (data->name_owner_id);
+      data->name_owner_id = 0;
+    }
+
+  g_clear_object (&data->connection);
+}
+
 static void
 authorize_uri_data_free (gpointer user_data)
 {
   AuthorizeUriData *data = user_data;
 
+  /* If the cancellable is triggered, we couldn't disconnect in the callback
+   * without deadlocking. Either way, this function is only called after
+   * the D-Bus bits are released and drop their references on the task.
+   */
   if (data->cancellable != NULL && data->cancellable_id != 0)
-    g_cancellable_disconnect (data->cancellable, data->cancellable_id);
+    {
+      g_cancellable_disconnect (data->cancellable, data->cancellable_id);
+      g_clear_object (&data->cancellable);
+    }
 
-  g_clear_object (&data->cancellable);
-  g_clear_object (&data->collection);
   g_clear_pointer (&data->request_uri, g_free);
   g_free (data);
 }
 
 static void
-authorize_uri_cancelled (GCancellable *cancellable,
-                         GTask        *task)
+oauth2_handler_dbus_method_call (GDBusConnection       *connection,
+                                 const char            *sender,
+                                 const char            *object_path,
+                                 const char            *interface_name,
+                                 const char            *method_name,
+                                 GVariant              *parameters,
+                                 GDBusMethodInvocation *invocation,
+                                 gpointer               user_data)
 {
-  AuthorizeUriData *data = g_task_get_task_data (task);
+  GTask *task = G_TASK (user_data);
 
-  if (!data->completed)
+  if (g_strcmp0 (method_name, "Response") == 0)
     {
-      g_task_return_error_if_cancelled (task);
-      g_signal_handlers_disconnect_by_data (data->collection, task);
-      data->completed = TRUE;
+      const char *client_id = NULL;
+      const char *response = NULL;
+      GError *error = NULL;
+
+      g_variant_get (parameters, "(&s&s)", &client_id, &response);
+      if (g_uri_is_valid (response, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, &error))
+        {
+          g_debug ("Received OAuth2 response for client ID \"%s\"", client_id);
+
+          g_dbus_method_invocation_return_value (invocation, NULL);
+          g_task_return_pointer (task, g_strdup (response), g_free);
+        }
+      else
+        {
+          g_debug ("Received invalid OAuth2 response for client ID \"%s\"", client_id);
+
+          g_dbus_method_invocation_return_error (invocation,
+                                                 G_DBUS_ERROR,
+                                                 G_DBUS_ERROR_INVALID_ARGS,
+                                                 "Invalid URI \"%s\"",
+                                                 response);
+          g_task_return_error (task, g_steal_pointer (&error));
+        }
+
+      authorize_uri_task_complete (NULL, task);
+      return;
     }
+
+  g_dbus_method_invocation_return_error (invocation,
+                                         G_DBUS_ERROR,
+                                         G_DBUS_ERROR_UNKNOWN_METHOD,
+                                         "Unknown method %s on %s",
+                                         method_name,
+                                         interface_name);
 }
 
-static void
-authorize_uri_check_response_cb (GObject      *object,
-                                 GAsyncResult *result,
-                                 gpointer      user_data)
-{
-  g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  AuthorizeUriData *data = g_task_get_task_data (task);
-  g_autofree char *response_uri = NULL;
-  GError *error = NULL;
-
-  /* A lookup may resolve after the task has completed or cancelled.
-   */
-  if (data->completed)
-    return;
-
-  response_uri = secret_password_lookup_finish (result, &error);
-  if (response_uri != NULL)
-    {
-      g_task_return_pointer (task, g_steal_pointer (&response_uri), g_free);
-      g_signal_handlers_disconnect_by_data (data->collection, task);
-      data->completed = TRUE;
-    }
-  else if (error != NULL)
-    {
-      g_task_return_error (task, g_steal_pointer (&error));
-      g_signal_handlers_disconnect_by_data (data->collection, task);
-      data->completed = TRUE;
-    }
-}
-
-static void
-authorize_uri_collection_changed_cb (SecretCollection *collection,
-                                     GParamSpec       *pspec,
-                                     GTask            *task)
-{
-  GoaOAuth2Provider *self = g_task_get_source_object (task);
-  GCancellable *cancellable = g_task_get_cancellable (task);
-
-  secret_password_lookup (&oauth2_schema,
-                          cancellable,
-                          (GAsyncReadyCallback) authorize_uri_check_response_cb,
-                          g_object_ref (task),
-                          "goa-oauth2-client", goa_oauth2_provider_get_client_id (self),
-                          NULL);
-}
+static const GDBusInterfaceVTable oauth2_handler_dbus_vtable =
+  {
+    oauth2_handler_dbus_method_call,
+    NULL,
+    NULL
+  };
 
 static void
 authorize_uri_launch_uri_cb (GObject      *object,
@@ -1100,53 +1205,56 @@ authorize_uri_launch_uri_cb (GObject      *object,
                              gpointer      user_data)
 {
   g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  AuthorizeUriData *data = g_task_get_task_data (task);
   GError *error = NULL;
 
   if (!g_app_info_launch_default_for_uri_finish (result, &error))
     {
-      g_signal_handlers_disconnect_by_data (data->collection, task);
       g_task_return_error (task, g_steal_pointer (&error));
+      authorize_uri_task_complete (NULL, task);
     }
 }
 
 static void
-authorize_uri_reset_collection_cb (SecretCollection *collection,
-                                   GAsyncResult     *result,
-                                   gpointer          user_data)
+on_oauth2_bus_acquired (GDBusConnection *connection,
+                        const char      *name,
+                        gpointer         user_data)
 {
-  g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  GCancellable *cancellable = g_task_get_cancellable (task);
+  static size_t guard = 0;
+  GTask *task = G_TASK (user_data);
   AuthorizeUriData *data = g_task_get_task_data (task);
   GError *error = NULL;
 
-  secret_password_clear_finish (result, &error);
-  if (error != NULL)
+  if (g_once_init_enter (&guard))
+    {
+      oauth2_handler_dbus_info = g_dbus_node_info_new_for_xml (oauth2_handler_dbus_xml, NULL);
+      g_once_init_leave (&guard, 1);
+    }
+
+  data->connection = g_object_ref (connection);
+  data->register_object_id =
+    g_dbus_connection_register_object (data->connection,
+                                       OAUTH2_DBUS_HANDLER_PATH,
+                                       oauth2_handler_dbus_info->interfaces[0],
+                                       &oauth2_handler_dbus_vtable,
+                                       g_object_ref (task),
+                                       g_object_unref,
+                                       &error);
+
+  if (data->register_object_id == 0)
     {
       g_task_return_error (task, g_steal_pointer (&error));
-      return;
+      authorize_uri_task_complete (NULL, task);
     }
+}
 
-  /* The signal handler holds a final reference on the task object, until
-   * the secret is added to the collection or an error occurs.
-   */
-  g_signal_connect_data (data->collection,
-                         "notify::items",
-                         G_CALLBACK (authorize_uri_collection_changed_cb),
-                         g_object_ref (task),
-                         (GClosureNotify)g_object_unref,
-                         G_CONNECT_DEFAULT);
-
-  /* If the operation is cancelled, the callback will complete the task and
-   * disconnect the signal handler holding the final reference on the task.
-   */
-  if (cancellable != NULL)
-    {
-      data->cancellable = g_object_ref (cancellable);
-      data->cancellable_id = g_cancellable_connect (cancellable,
-                                                    G_CALLBACK (authorize_uri_cancelled),
-                                                    task, NULL);
-    }
+static void
+on_oauth2_name_acquired (GDBusConnection *connection,
+                         const char      *name,
+                         gpointer         user_data)
+{
+  GTask *task = G_TASK (user_data);
+  GCancellable *cancellable = g_task_get_cancellable (task);
+  AuthorizeUriData *data = g_task_get_task_data (task);
 
   if ((data->flags & GOA_AUTH_FLOW_DO_NOT_LAUNCH_URI) == 0)
     {
@@ -1159,69 +1267,25 @@ authorize_uri_reset_collection_cb (SecretCollection *collection,
 }
 
 static void
-authorize_uri_get_collection_cb (SecretService *service,
-                                 GAsyncResult  *result,
-                                 gpointer       user_data)
+on_oauth2_name_lost (GDBusConnection *connection,
+                     const char      *name,
+                     gpointer         user_data)
 {
-  g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  GoaOAuth2Provider *self = g_task_get_source_object (task);
-  GCancellable *cancellable = g_task_get_cancellable (task);
-  AuthorizeUriData *data = g_task_get_task_data (task);
-  GError *error = NULL;
+  GTask *task = G_TASK (user_data);
 
-  g_return_if_fail (G_IS_TASK (task));
+  if (connection == NULL)
+    g_warning ("%s(): Failed to connect to the session bus", G_STRFUNC);
+  else
+    g_warning ("%s(): Failed to own %s on the session bus", G_STRFUNC, name);
 
-  data->collection = secret_collection_for_alias_finish (result, &error);
-  if (data->collection == NULL)
-    {
-      if (error == NULL)
-        {
-          g_set_error_literal (&error,
-                               GOA_ERROR,
-                               GOA_ERROR_FAILED,
-                               _("Failed to get session keyring"));
-        }
-
-      g_task_return_error (task, g_steal_pointer (&error));
-      return;
-    }
-
-  /* Ensure there's no old authorization URI hanging around.
-   */
-  secret_password_clear (&oauth2_schema,
-                         cancellable,
-                         (GAsyncReadyCallback) authorize_uri_reset_collection_cb,
-                         g_object_ref (task),
-                         "goa-oauth2-client", goa_oauth2_provider_get_client_id (self),
-                         NULL);
+  g_task_return_new_error_literal (task,
+                                   GOA_ERROR,
+                                   GOA_ERROR_FAILED,
+                                   _("Service not available"));
+  authorize_uri_task_complete (NULL, task);
 }
 
-static void
-authorize_uri_get_service_cb (GObject      *object,
-                              GAsyncResult *result,
-                              gpointer      user_data)
-{
-  g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  GCancellable *cancellable = g_task_get_cancellable (task);
-  g_autoptr(SecretService) service = NULL;
-  GError *error = NULL;
-
-  g_return_if_fail (G_IS_TASK (task));
-
-  service = secret_service_get_finish (result, &error);
-  if (service == NULL)
-    {
-      g_task_return_error (task, g_steal_pointer (&error));
-      return;
-    }
-
-  secret_collection_for_alias (service,
-                               SECRET_COLLECTION_SESSION,
-                               SECRET_COLLECTION_LOAD_ITEMS,
-                               cancellable,
-                               (GAsyncReadyCallback) authorize_uri_get_collection_cb,
-                               g_object_ref (task));
-}
+/* ---------------------------------------------------------------------------------------------------- */
 
 /*< private >
  * goa_oauth2_provider_authorize_uri:
@@ -1263,10 +1327,23 @@ goa_oauth2_provider_authorize_uri (GoaOAuth2Provider   *provider,
   g_task_set_source_tag (task, goa_oauth2_provider_authorize_uri);
   g_task_set_task_data (task, data, authorize_uri_data_free);
 
-  secret_service_get (SECRET_SERVICE_LOAD_COLLECTIONS | SECRET_SERVICE_OPEN_SESSION,
-                      cancellable,
-                      (GAsyncReadyCallback) authorize_uri_get_service_cb,
-                      g_object_ref (task));
+  if (cancellable != NULL)
+    {
+      data->cancellable = g_object_ref (cancellable);
+      data->cancellable_id = g_cancellable_connect (cancellable,
+                                                    G_CALLBACK (authorize_uri_task_complete),
+                                                    task,
+                                                    NULL);
+    }
+
+  data->name_owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                                        OAUTH2_DBUS_HANDLER_NAME,
+                                        G_BUS_NAME_OWNER_FLAGS_NONE,
+                                        on_oauth2_bus_acquired,
+                                        on_oauth2_name_acquired,
+                                        on_oauth2_name_lost,
+                                        g_object_ref (task),
+                                        g_object_unref);
 }
 
 /*< private >
