@@ -55,9 +55,12 @@ goa_utils_check_duplicate (GoaClient              *client,
       const gchar *provider_type_from_object;
 
       account = goa_object_peek_account (object);
-      interface = (*func) (object);
-      if (interface == NULL)
-        continue;
+      if (func != NULL)
+        {
+          interface = (*func) (object);
+          if (interface == NULL)
+            continue;
+        }
 
       provider_type_from_object = goa_account_get_provider_type (account);
       if (g_strcmp0 (provider_type_from_object, provider_type) != 0)
@@ -718,7 +721,7 @@ goa_utils_normalize_url (const char  *base_uri,
     }
 
   uri = g_uri_parse (uri_string, G_URI_FLAGS_ENCODED | G_URI_FLAGS_PARSE_RELAXED, NULL);
-  if (uri == NULL || g_uri_get_host (uri) == NULL || g_uri_get_host (uri)[0] == '\0')
+  if (uri == NULL)
     return NULL;
 
   if (uri_ref != NULL)
